import gurobipy as g
import matplotlib.pyplot as plt
import matplotlib.patches as patches


__author__ = 'tonda'

n = 5
orders = {
    0: {'r': 20, 'd': 45, 'p': 15},
    1: {'r': 4, 'd': 30, 'p': 19},
    2: {'r': 5, 'd': 80, 'p': 20},
    3: {'r': 17, 'd': 70, 'p': 8},
    4: {'r': 27, 'd': 66, 'p': 7},
}

# model
m = g.Model()

s = [m.addVar(vtype=g.GRB.CONTINUOUS, lb=0) for i in range(n)]
x = {}
for i in range(n):
    for j in range(i):
        x[i, j] = m.addVar(vtype=g.GRB.BINARY)

Cmax = m.addVar(vtype=g.GRB.CONTINUOUS, obj=1)

for i in range(n):
    m.addConstr(s[i] + orders[i]['p'] <= Cmax)
    m.addConstr(s[i] >= orders[i]['r'])
    m.addConstr(s[i] + orders[i]['p'] <= orders[i]['d'])

#M = sum([orders[i]['p']+orders[i]['r'] for i in range(n)])
M = max([orders[i]['d'] for i in range(n)])
for i in range(n):
    for j in range(i):
        m.addConstr(s[i] + orders[i]['p'] <= s[j] + M*x[i, j])
        m.addConstr(s[j] + orders[j]['p'] <= s[i] + M*(1-x[i, j]))

m.optimize()

print('solution:')
for i in range(n):
    print('order {} starts baking at: {}'.format(i, int(round(s[i].x))))

# visualization
hatches = ['\\', '/', 'o', '//', '*']
colors = ['r', 'b', 'g', 'y', 'orange']
fig, ax = plt.subplots(1, figsize=(13, 3))
for i in range(n):
    ax.add_patch(
        patches.Rectangle((int(round(s[i].x)), 0), orders[i]['p'], 0.3, hatch=hatches[i], color=colors[i])
    )
    ax.annotate('order {}'.format(i), (int(round(s[i].x)), 0.15), color='white', weight='bold')
plt.xlim([0, 90])
plt.show()