import gurobipy as g
import matplotlib.pyplot as plt
import numpy as np

# Board size.
n = 19   # 20

m = g.Model()

x = {}
for i in range(0, n + 2):
    for j in range(0, n + 2):
        x[i, j] = m.addVar(vtype=g.GRB.BINARY, name="x[{0},{1}]".format(i, j))

k = {}
for i in range(1, n + 1):
    for j in range(1, n + 1):
        k[i, j] = m.addVar(lb=0, vtype=g.GRB.INTEGER, name="k[{0},{1}]".format(i, j))

m.update()

for i in range(1, n + 1):
    for j in range(1, n + 1):
        m.addConstr(x[i, j] + x[i, j+1] + x[i, j-1] + x[i-1, j] + x[i + 1, j] == 1 + 2*k[i, j],
                    name="4ngh[{0},{1}]".format(i, j))

for i in range(0, n + 2):
    m.addConstr(x[0, i] + x[i, 0] + x[n + 1, i] + x[i, n + 1] == 0,
                name="outboard[{0}]".format(i))

m.setObjective(g.quicksum(x.values()), sense=g.GRB.MINIMIZE)

m.optimize()

m.write("fivers.lp")

X = np.array([[x[i, j].x for j in range(1, n+1)] for i in range(1, n+1)])
clicks = np.argwhere(X >= 0.5)

plt.imshow(X, interpolation='none')
plt.scatter(clicks[:, 0], clicks[:, 1], c='red')
plt.gca().set_xticks(np.arange(-0.5, n, 1))
plt.gca().set_xticklabels([])
plt.gca().set_yticks(np.arange(-0.5, n, 1))
plt.gca().set_yticklabels([])
plt.grid()
plt.show()
