import gurobi.*;

public class Fivers {
	public static void main(String[] args) throws Exception {
     int n = 5;

     GRBEnv env = new GRBEnv();
     GRBModel m = new GRBModel(env);

     // The number of running base load power plants.
     GRBVar[][] x = new GRBVar[n][n];
     GRBVar[][] z = new GRBVar[n][n];

     for(int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
	    x[i][j] = m.addVar(0.0, 1.0, 1.0, GRB.BINARY, "x");
	    z[i][j] = m.addVar(0.0, 2.0, 0, GRB.INTEGER, "z");
	}
     }

     // Integrate new variables into model.
     m.update();

     // Constraint
     for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
	    GRBLinExpr expr = new GRBLinExpr();
	    expr.addTerm(1.0, x[i][j]);
	    if(i != n - 1) {
		expr.addTerm(1.0, x[i + 1][j]);
	    }

	    if(i != 0) {
		expr.addTerm(1.0, x[i - 1][j]);
	    }

	    if(j != n - 1) {
		expr.addTerm(1.0, x[i][j + 1]);
	    }

	    if(j != 0) {
		expr.addTerm(1.0, x[i][j - 1]);
	    }

	    expr.addTerm(-2.0, z[i][j]);
	    m.addConstr(expr, GRB.EQUAL, 1.0, "cell_" + i + "_" + j);
	}
     }

     m.write("mod.lp");

     // Solve the model. 
     m.optimize();

     System.out.println("Objective value: " + m.get(GRB.DoubleAttr.ObjVal));
     for (int i = 0; i < n; i++) {
	for (int j = 0; j < n; j++) {
	    System.out.println("x_" + (i + 1) + "_" + (j + 1) + " = "+ x[i][j].get(GRB.DoubleAttr.X));
	}
     }
     }
}
